var FRAMEDBG = false;
var INITDBG = true;
class PcCameraControl {
  constructor() {
  }

  static useDSLR()
  {
    return !PcConfig.scenario.useWebcam;
  }
  static init(videoMode = false) {
    this.liveViewStarted = false;
    this.fileToRecord = "";
    this.webcamActive = false;

    if (PcConfig.useElectron()) { // electron application
      this.process = require('electron').remote.require("child_process");
      this.fs = require('electron').remote.require("fs");
      this.path = require('electron').remote.require("path");

    }

    // constants declaration, Display Aspect Ratios (DAR) for picture and video modes
    const PIC_DAR_NUM = 3;
    const PIC_DAR_DEN = 2;
    const VID_DAR_NUM = 16;
    const VID_DAR_DEN = 9;

    // electron DSLR resolutions:
    // picture mode
    const ELECTRON_DSLR_W = 640; // NOTE: when we use a DSLR under electron, whatever mode (picture, video) we cannot go beyond a width of 640 px 
    const ELECTRON_DSLR_H = ELECTRON_DSLR_W * PIC_DAR_DEN / PIC_DAR_NUM; // in picture, dar is 3/2 => ~= 426-ish
    // const ELECTRON_DSLR_H = "424"; // legacy value, to delete. note: dar = 1.509, a bit more than with 426 ^^^

    // video mode
    const ELECTRON_DSLR_W_REC = ELECTRON_DSLR_W;
    const ELECTRON_DSLR_H_REC = ELECTRON_DSLR_W_REC * VID_DAR_DEN / VID_DAR_NUM; // video mode: dar = 16/9 => 360px

    // electron webcam resolutions:
    // NOTE: We want a fixed height of 1080 px
    // picture mode
    const ELECTRON_WEBCAM_H = 1080;
    const ELECTRON_WEBCAM_W = ELECTRON_WEBCAM_H * PIC_DAR_NUM / PIC_DAR_DEN; // pic mode: dar = 3/2 => 1620

    // video mode 
    const ELECTRON_WEBCAM_H_REC = ELECTRON_WEBCAM_H;
    const ELECTRON_WEBCAM_W_REC = ELECTRON_WEBCAM_H_REC * VID_DAR_NUM / VID_DAR_DEN; // video mode: dar = 16/9 => 1920px

    // webview preview webcam resolutions:
    // NOTE: we want a fixed width of 1280px.... AND pic DAR = 16/9 instead of regular 3/2 (FIXME: IS IT BECAUSE SOME WEBCAMS ARE 16/9 ONLY?)
    // picture  mode
    const WEBVIEW_WEBCAM_W = 1280;
    const WEBVIEW_WEBCAM_H = WEBVIEW_WEBCAM_W * VID_DAR_DEN / VID_DAR_NUM ; // dodgy pic mode: dar = 16/9 (!) => 720px

    // video mode
    const WEBVIEW_WEBCAM_W_REC = WEBVIEW_WEBCAM_W; // in video mode with webview, video res = pic res
    const WEBVIEW_WEBCAM_H_REC = WEBVIEW_WEBCAM_H;

    let FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC, img_w, img_h, cam_w, cam_h, cam_w_rec, cam_h_rec;
    let arr;

    PcConfig.dbg(INITDBG, "PcCameraControl init(): useElectron? ", PcConfig.useElectron());
    PcConfig.dbg(INITDBG, "useWebcam? " + PcConfig.scenario.useWebcam);
    PcConfig.dbg(INITDBG, "useDSLR? ", this.useDSLR());

    if (PcConfig.useElectron()) { // final lengths against modes {electron,webview} x {dslr, webcam}
      if (this.useDSLR()) {
        arr = [ ELECTRON_DSLR_W, ELECTRON_DSLR_H, ELECTRON_DSLR_W_REC, ELECTRON_DSLR_H_REC ] ;

      } else if (PcConfig.scenario.useWebcam) {    
        arr = [ ELECTRON_WEBCAM_W, ELECTRON_WEBCAM_H, ELECTRON_WEBCAM_W_REC, ELECTRON_WEBCAM_H_REC ] ;
      }
    } else { // webview, no DSLR, so unique resoltution available  
      arr = [ WEBVIEW_WEBCAM_W, WEBVIEW_WEBCAM_H, WEBVIEW_WEBCAM_W_REC, WEBVIEW_WEBCAM_H_REC ] ;
    }

    // PcConfig.dbg(true, "arr: ", arr);
    // round our lengths and convert them to strings
    [ FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC ] = arr.map(i  => { return String(Math.floor(i)); } ); // round values, convert to string (legacy)

    [img_w, img_h] = [FINAL_W, FINAL_H];

    if (PcConfig.scenario.cameraOrientation == "portrait" || PcConfig.scenario.cameraOrientation == "portraitInverted") {
      [cam_w, cam_h, cam_w_rec, cam_h_rec] = [FINAL_H, FINAL_W, FINAL_H_REC, FINAL_W_REC];
    } else {
      [cam_w, cam_h, cam_w_rec, cam_h_rec] = [FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC];
    }
      
    PcConfig.dbg(INITDBG, "cam_h: ", cam_h, "cam_w: ", cam_w, "cam_h_rec: ", cam_h_rec, "cam_w_rec: ", cam_w_rec);

    this.cameraWidth = cam_w;
    this.cameraHeight = cam_h;
    this.cameraWidthWhileRecording = cam_w_rec;
    this.cameraHeightWhileRecording = cam_h_rec;
    this.imgWidth = img_w;
    this.imgHeight = img_h;
    this.img = new Image(this.imgWidth, this.imgHeight);


    this.liveViewTimeout = null;
    this.isShooting = false;

    this.video = document.createElement("video");

    this.videoMode = videoMode;
    this.videoRecording = false;

    this.canvas = document.createElement('canvas');
    this.canvas.width = this.cameraWidth;
    this.canvas.height = this.cameraHeight;

    this.context = this.canvas.getContext('2d');

    this.sleep = ms => new Promise(resolve => setTimeout(resolve, ms));

    this.startStamp = 0;
    this.flashStamp = [];


    this.initLiveView();
  }

  static initLiveView(fOnDo = null) {
    PcConfig.dbg(LEGACYDBG, "init liveview: useDSLR? " + this.useDSLR() + " because useWebCam? " + PcConfig.scenario.useWebcam);

    if (this.useDSLR()) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Init";
      PcConfig.myProcessExec(cmd, (error, stdout, stderr) => // Recorder Init (liveView)
      {
        if (fOnDo) fOnDo();
      }
      );
    }
    else {
      if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
        navigator.mediaDevices.getUserMedia({ video: { width: this.imgWidth, height: this.imgHeight } }).then((stream) => {
          this.video.srcObject = stream;
          this.video.play();
          this.webcamActive = true;
        }, () => { PcConfig.dbg(LEGACYDBG, "WEBCAM NOT DETECTED !") });
      }
      else {
        this.webcamActive = false;
        PcConfig.dbg(true, "GetUserMedia didn't work, ARGH");
      }
    }
  }

  static startLiveView() {
    PcConfig.dbg(LEGACYDBG, "enter startLiveView");

    if (this.useDSLR() && !this.liveViewStarted) {
      PcConfig.dbg(LEGACYDBG, "DO StartLiveView");

      this.liveViewStarted = true;

      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StartLiveView";
      PcConfig.myProcessExec(cmd, () => // Recorder StartLiveView
      {
        this.liveViewTimeout = setTimeout(() => PcCameraControl.img.src = "http://localhost:8083/?x=" + Math.random(), 500);
      }
      );

    }
  }

  static stopLiveView() {
    if (this.useDSLR()) {
      this.liveViewStarted = false;

      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StopLiveView";

      clearTimeout(this.liveViewTimeout);
      this.img.src = "";
      PcConfig.myProcessExec(cmd); // recorder StopLiveView
    }
  }

  static autofocus() {
    if (this.useDSLR()) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Autofocus";
      PcConfig.myProcessExec(cmd); // Recorder Autofocus
    }
  }

  static shoot() {
    //PcCameraControl.stopLiveView();
    if (this.useDSLR() == false) {
      setTimeout(() => { this.doShoot() }, 1750);
    }
    else {
      this.doShoot();
    }

    //setTimeout(()=>PcCameraControl.startLiveView(), 2000);
  }

  static startVideoRecording() {
    if (this.videoRecording) return;

    if (this.useDSLR()) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StartRecording";
      PcConfig.myProcessExec(cmd, () => { // Recorder StartRecording video
        this.videoRecording = true;
        PcConfig.dbg(LEGACYDBG, "recording video " + Date.now() / 1000);
        while (this.flashStamp.length) { this.flashStamp.pop(); }
        this.startStamp = Date.now() / 1000;
      });
    }
  }

  static stopVideoRecording() {
    if (this.useDSLR()) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StopRecording";
      PcConfig.myProcessExec(cmd, () => this.videoRecording = false); // Recorder StopRecording video
    }
  }

  static transferVideoRecording(fOnTransfer) {
    if (this.useDSLR()) {
      let filename = (new Date().getTime() + Math.floor((Math.random() * 10000) + 1)).toString(16) + ".mov";
      let fileToRecord = PcConfig.global.recordedFolder + filename;

      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Transfer " + fileToRecord;

      PcConfig.myProcessExec("del /q " + PcConfig.global.recordedFolderForDelete, () => { }); // Delete recorderFolderForDelete (del /q ...)

      PcConfig.myProcessExec(cmd, () =>  // Recorder Transfer Video Recording)
      {
        PcConfig.setMemory('recordedVideoFile', fileToRecord);
        PcConfig.setMemory('recordedVideoFileName', filename);
        if (fOnTransfer) fOnTransfer();
      });
    }
    else {
      PcConfig.setMemory('recordedVideoFile', PcConfig.host + 'ressources/videodemo.mp4');
      if (fOnTransfer) fOnTransfer();
    }
  }

  static doBoommerangShoot() {
    let canvas = document.createElement('canvas');
    canvas.width = this.video.videoWidth;
    canvas.height = this.video.videoHeight;
    canvas.getContext('2d').drawImage(this.video, 0, 0, this.video.videoWidth, this.video.videoHeight);
    PcCaptureSession.addFrame(canvas, true);
  }

  static doShoot() {
    PcConfig.dbg(LEGACYDBG, "DO SHOOT");
    let fs = require('electron').remote.require("fs");
    if(this.useDSLR()) {
      var n = PcCaptureSession.shot.length;
      var uniqid = (new Date().getTime() + Math.floor((Math.random() * 10000) + 1)).toString(16);
      var name = "photo" + uniqid + "_" + n + ".jpg";
      var file = PcConfig.global.recordedFolder + name;
      var hostedFile = PcConfig.host + PcConfig.global.recordedFolderForSave + name;
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /c " + file;

      PcConfig.myProcessExec(cmd, async () => { // Recorder /c (shoot photo)
        var find = false;
        var count = 0;
        var maxCount = 100;

        do {

          if (fs.existsSync(file)) { find = true; }
          count++;
          await this.sleep(100);
        } while (!find && count <= maxCount)

        if (!find) {
          PcConfig.dbg(LEGACYDBG, "PHOTO NOT FOUND, RESTARTING");
          PcConfig.restartAll();
        }
        else {
          PcConfig.dbg(LEGACYDBG, "PHOTO FOUND");

          var not_ready = true;
          do {
            PcConfig.dbg(false, "nr ? ", not_ready);
            try {
              fs.readFileSync(file, 'utf-8'); //did you know? this intricate loop could block restartAll before fix for bug #6
              not_ready = false;
            }
            catch (error) {
              //console.log (" ex" , error); // usually ebusy, not a problem, let's wait some more
              not_ready = true;
            }
          } while (not_ready == true);
          PcCaptureSession.addFrame(file);
          PcCaptureSession.nextShot();
        }
      });

    }
    else if(this.webcamActive) // webcam active (electron with useWebcam = true, or web view where webcam was detected)
    {
      let canvas = document.createElement('canvas');
      canvas.width = this.video.videoWidth;
      canvas.height = this.video.videoHeight;
      canvas.getContext('2d').drawImage(this.video, 0, 0, this.video.videoWidth, this.video.videoHeight);
      PcCaptureSession.addFrame(canvas, true);
      PcCaptureSession.nextShot();
    }
    else //webcam doesn't exist (electron with useWebcam = false, or web view where webcam was not detected)
    {
      let canvas = document.createElement('canvas');
      canvas.width = 800;
      canvas.height = 450;
      canvas.fillStyle = "black";
      canvas.getContext('2d').fillRect(0, 0, canvas.width, canvas.height);
      PcCaptureSession.addFrame(canvas, true);
      PcCaptureSession.nextShot();
    }

  }

  static exitDSLR() {
    if (this.useDSLR()) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Exit";
      PcConfig.myProcessExec(cmd); // Recorder Exit
    }
  }

  static exitDSLRStrong(fOnDo = null) {
    if (this.useDSLR()) {
      let cmd = "";

      this.stopVideoRecording();
      this.stopLiveView();

      cmd = 'Taskkill /IM Photocabine.Process.exe /F /T';
      setTimeout(() => {
        //console.log("taskkill Photocabine.process.exe");
        PcConfig.myProcessExec(cmd, () => { // Taskkill PhotocabineProcess.exe 
          if (fOnDo) fOnDo();
        });
      }, 1000);
    }
  }

  static getCurrentFrame(destW, destH) {
    var cameraWidth, cameraHeight, cameraX, cameraY;
    var destCW, destCH, destX, destY;
    var oW, oH, oX, oY;

    if (PcConfig.getMemory("currentFormat").animation == "boomerang") {
      cameraHeight = this.cameraHeightWhileRecording;
    }
    else {
      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraHeight = this.cameraHeightWhileRecording;
      }
      else {
        cameraHeight = this.cameraHeight;
      }
    }

    if (PcConfig.scenario.mode == "photobooth" && !PcConfig.screen.format.wideCapture && !PcConfig.getMemory("currentFormat").autocrop) {
      cameraWidth = Math.min(this.cameraWidth, cameraHeight);
      cameraHeight = cameraWidth;
      cameraX = (this.cameraWidth - cameraWidth) / 2;

      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraY = 0;
      }
      else {
        cameraY = (this.cameraHeight - cameraHeight) / 2;
      }
    }
    else if (PcConfig.scenario.mode == "videobooth" || PcConfig.scenario.mode == "videokiosk") {
      cameraWidth = this.cameraWidth;
      cameraX = 0;
      cameraY = 0;

      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraHeight = this.cameraHeightWhileRecording;
      }
      else {
        cameraHeight = this.cameraHeight;
      }
    }
    else {
      cameraWidth = this.cameraWidth;
      cameraX = 0;
      cameraY = 0;
    }

    if ((destW / destH) < (cameraWidth / cameraHeight)) {
      oH = cameraHeight;
      oW = cameraHeight * destW / destH;
      oY = cameraY;
      oX = cameraX + (cameraWidth - oW) / 2;
    }
    else {
      oW = cameraWidth;
      oH = cameraWidth * destH / destW;
      oY = cameraY + (cameraHeight - oH) / 2;
      oX = cameraX;
    }

    let canvas = document.createElement('canvas');
    canvas.width = destW;
    canvas.height = destH;
    let context = canvas.getContext('2d');

    try {
      PcConfig.dbg(FRAMEDBG, "getCurrentFrame: useDSLR? " + this.useDSLR() + " webcamActive? " + this.webcamActive);
      if (this.useDSLR() || this.webcamActive) {
        let source;
        if (this.useDSLR()) {
          source = this.img;
        } else if (this.webcamActive) {
          source = this.video;
        }
        if (PcConfig.scenario.cameraOrientation == "portrait" || PcConfig.scenario.cameraOrientation == "portraitInverted") {
          let target;
          let ndw;
          let ndh;

          let newcanvas = document.createElement('canvas');
          let newcontext = newcanvas.getContext('2d');

          if (this.useDSLR()) {
            PcConfig.dbg(FRAMEDBG, "useDSLR!");
            target = PcCameraControl.cloneCanvas(source);            
                        
          } else if (this.webcamActive) {
            PcConfig.dbg(FRAMEDBG, "webcamActive!");
            let videoCanvas = document.createElement('canvas');
            let videoContext = videoCanvas.getContext('2d');
            videoCanvas.width = this.cameraHeight;
            videoCanvas.height = this.cameraWidth;
            target = videoCanvas;

            PcConfig.dbg(FRAMEDBG, "Portrait Webcam DI1");
            videoContext.drawImage(source, 0, 0);
          }
          ndw = target.width;
          ndh = target.height;

          newcontext.clearRect(0, 0, newcanvas.width, newcanvas.height);
          newcanvas.width = ndh;
          newcanvas.height = ndw;
          newcontext.save();
          newcontext.translate(newcanvas.width / 2, newcanvas.height / 2);

          if (PcConfig.scenario.cameraOrientation == "portrait") {
            newcontext.rotate(-90 * Math.PI / 180);
          } else { // portrait inverted
            newcontext.rotate(90 * Math.PI / 180);
          }
          PcConfig.dbg(FRAMEDBG, "Portrait Common DI2");
          newcontext.drawImage(target, -ndw / 2, -ndh / 2, ndw, ndh);
          newcontext.restore();
          PcConfig.dbg(FRAMEDBG, "Portrait Common DI3");
          context.drawImage(newcanvas, oX, oY, oW, oH, 0, 0, destW, destH);
        } else {  // landscape  
          PcConfig.dbg(FRAMEDBG, "landscape");
          context.drawImage(source, oX, oY, oW, oH, 0, 0, destW, destH);
        }
      } else {  // neither dsl or webcam active
        context.fillStyle = "#333333";
        context.fillRect(0, 0, canvas.width, canvas.height);
        context.fillStyle = "#888888";
        context.scale(-1, 1);
        context.font = "16px Arial";
        context.textAlign = "center";
        context.fillText("(webcam not detected)", -canvas.width / 2, canvas.height / 2);
      }
    }
    catch (error) {
      console.log("ERROR getCurrentFrame failed with EXCEPTION:" + error);
      PcCameraControl.img.src = "http://localhost:8083/?x=" + Math.random();
    }

    context = null;

    return canvas;
  }

  static cloneCanvas(oldCanvas) {
    var newCanvas = document.createElement('canvas');
    var context = newCanvas.getContext('2d');

    newCanvas.width = oldCanvas.width;
    newCanvas.height = oldCanvas.height;

    context.drawImage(oldCanvas, 0, 0);

    return newCanvas;
  }

  static deleteOldShoots() {
    if (!this.useDSLR()) return;

    const directory = PcConfig.global.recordedFolder;

    let fs = require('electron').remote.require("fs");

    fs.readdir(directory, (err, files) => {
      if (err) throw err;

      let path = require('electron').remote.require("path");

      for (const file of files) {
        fs.unlink(path.join(directory, file), err => {
          if (err) throw err;
        });
      }
    });
  }

}
