class PcPhoto extends React.Component
{
  constructor(props)
  {
    super(props);

    this.canvas = React.createRef();
    this.canvasContainer = React.createRef();
    this.label = React.createRef();
    this.frames = []
    this.currentFrame = 0;
    this.interval = null;
    this.timerInterval = 0;
    this.intervalGif4 = 500;
    this.intervalBoomerang = 100;
    this.animated = false;
    this.filter = props.filter;
    this.started=false;
  }

  updateCanvas()
  {
    if (!this.frames[this.currentFrame])
    {
      this.currentFrame = 0;
      return;
    }

    let canvas = this.canvas.current;
    let ctx = canvas.getContext('2d');

    canvas.width = this.frames[this.currentFrame].width;
    canvas.height = this.frames[this.currentFrame].height;

    ctx.drawImage(this.frames[this.currentFrame], 0, 0);

    let ratio =0.85;
    let cw, ch;

    cw = parseInt(this.props.width*ratio);
    ch = parseInt(this.props.width*ratio * (this.frames[this.currentFrame].height/this.frames[this.currentFrame].width));

    if (ch>this.props.height*ratio)
    {
        ch = parseInt(this.props.height*ratio);
        cw = parseInt(ch * (this.frames[this.currentFrame].width/this.frames[this.currentFrame].height));
    }

    canvas.style.width = cw + "px";
    canvas.style.height = ch + "px";

    if (this.currentFrame>= this.frames.length-1)
    {
      this.currentFrame = 0;
    }
    else {
      this.currentFrame++;
    }
  }

  start()
  {
    if (this.animated) this.interval = setInterval(()=>{this.updateCanvas()}, this.timerInterval);
    this.started = true;
  }

  stop()
  {
    clearInterval(this.interval);
    this.started = false;
  }

  setFilter(filter)
  {
    let started = this.started;
    this.filter = filter;
    this.delete();
    this.currentFrame = 0;
    this.createFrames();
    this.updateCanvas();
    if (started) this.start();
  }

  createFrames()
  {
    var shots = [];
    for (let i=0; i<this.props.shot.length; i++) shots[i] = this.props.shot[i].frame[0].filtered[this.filter.id];

    if (this.props.preview)
    {
      let frame = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots);
      this.frames[0] = PcTemplates.createThumb([frame], this.props.width, this.props.width);
    }
    else {
      if (this.props.format.animation=="none")
      {
          this.frames[0] = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots);
      }
      else if (this.props.format.animation=="gif4")
      {
        this.frames[0] = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots.slice(0, shots.length));
        for (let i=1; i<4; i++)
        {
          setTimeout(()=>{
            this.frames[i] = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots.slice(i, shots.length));
          }, i*this.intervalGif4);
        }
        this.animated = true;
        this.timerInterval = this.intervalGif4;
      }
      else if (this.props.format.animation=="boomerang")
      {
        let framesNumber = this.props.shot[0].frame.length;
        for (let j=0; j<this.props.shot.length; j++) shots[j] = this.props.shot[j].frame[0].filtered[this.filter.id];
        this.frames[0] = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots);

        for (let i=1; i<framesNumber; i++)
        {
          setTimeout(()=>{
              for (let j=0; j<this.props.shot.length; j++) shots[j] = this.props.shot[j].frame[i].filtered[this.filter.id];
              this.frames[i] = PcTemplates.create(this.props.format, this.props.colors[0], this.props.backgrounds[0], this.props.foregrounds[0], shots);
          }, i*this.intervalBoomerang);

        }

        this.animated = true;
        this.timerInterval = this.intervalBoomerang;
      }
    }
  }

  componentDidMount()
  {
    this.createFrames();
    this.updateCanvas();
  }

  componentWillUnmount()
  {
    this.delete();
  }

  delete()
  {
    this.stop();

    for(let i=0; i<this.frames.length; i++)
    {
     this.frames[i] = null;
    }

    this.frames.splice(0);
  }

  render()
  {
    var shadow = parseInt(5*PcConfig.renderSize);
    var padding = this.props.padding;

    return [e('div',
      {
        ref:this.canvasContainer,
        key:"div1",
        style :
        {
            width:this.props.width,
            height:this.props.height*0.9,
            display:"table-cell",
            textAlign:"center",
            verticalAlign:"middle",
            padding:padding,
        }
      },
      e("canvas",
        {
          ref:this.canvas,
          className:"pcphotocanvas",
          style :
          {
            objectFit:"contain",
            borderStyle:this.props.preview ? "solid" : "none",
            borderWidth:this.props.preview ? PcConfig.scenario.buttonsBorderWidth : 0,
            borderColor:PcConfig.scenario.standardButtonBorderColor,
            borderRadius:this.props.preview ?  12 * PcConfig.renderSize : 0,
            boxShadow: shadow + "px " + shadow + "px " + shadow + "px 0px rgba(0, 0, 0, 0.5)",
          }
        }
      ),
      e('div', {style:{width:"100%", height:10*PcConfig.renderSize}}),
      e(PcLabel,
        {
          label:this.props.label,
          title:true,
          width:this.props.width,
          height:this.props.height*0.1,
          fontSize: this.props.preview ? PcConfig.scenario.fontSizeSmall : PcConfig.scenario.fontSizeMedium,
          color: PcConfig.scenario.fontColor,
          spacingV: this.props.preview ? 0*PcConfig.renderSize : 0*PcConfig.renderSize,
          showDelay:"0,1s"
        }
      )
    )];
  }

}
