class PcScreenLoading extends PcScreen
{
  constructor(props)
  {
    super(props);
  }

  getNumShoots()
  {
    var currentFormat = PcConfig.getMemory("currentFormat");
    var currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;

    if (currentFormat.animation=="none" || currentFormat.animation=="boomerang")
    {
      return currentPoses.length;
    }
    else
    {
      return 4;
    }
  }

  testShoots()
  {
    let shootsToWait = this.getNumShoots();

    if (PcConfig.getMemory("currentFormat").animation == "boomerang")
    {
      setTimeout(()=>PcCameraControl.transferVideoRecording(()=> {
          if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==true)
          {
            this.makeShootsFromWebcam();
          } else {
            this.makeShootsFromVideo();
          }
        }), 2000);
    }
    else
    {

      if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
      {
        setTimeout(()=>this.makeShootsFilters(), 500);
        return;
      }

      if (PcCaptureSession.shot.length<shootsToWait)
      {
        setTimeout(()=>this.testShoots(), 100);
      }
      else {
        setTimeout(()=>this.makeShootsFilters(), 500);
      }

    }

  }

  makeShootsFromWebcam()
  {
    let currentFormat = PcConfig.getMemory("currentFormat");
    let currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;
    let poses = currentPoses.length;

    for (let i = 0; i<poses; i++)
    {
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[6].canvas, true);
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[5].canvas, true);
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[4].canvas, true);
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[3].canvas, true);
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[2].canvas, true);
      PcCaptureSession.shot[i].addFrame(PcCaptureSession.shot[i].frame[1].canvas, true);
    }

    this.makeShootsFilters();
  }


  async makeShootsFromVideo()
  {
    console.log("makeshootsfromvideo " + Date.now()/1000);

    /*
    PcCameraControl.exitDSLRStrong(()=>{
      console.log("exit DSLR ok");
    });
    */

    function extractFrame(video, canvas, offset) {
      return new Promise((resolve, reject) => {
        video.onseeked = event => {
          let ctx = canvas.getContext('2d');
          ctx.drawImage(video, 0, 0, canvas.width, canvas.height);
          resolve(canvas);
          };
        video.currentTime = offset;
      });
    };

    let videoFile = PcConfig.getMemory('recordedVideoFile');
    let video = document.createElement ("video");
    let canvas = document.createElement('canvas');

    video.onloadedmetadata = async ()=>{
      canvas.width = video.videoWidth/1.5;
      canvas.height = video.videoHeight/1.5;

      let f = null;

      let currentFormat = PcConfig.getMemory("currentFormat");
      let currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;
      let poses = currentPoses.length;

      for (let i = 0; i<poses; i++)
      {
          for (let j=0; j<8; j++)
          {
            let t = PcCameraControl.flashStamp[i] + (j*1.5/7);
            f = await extractFrame(video, canvas, t);
            PcCaptureSession.addFrame(f, true);
          }

          for (let j=6; j>=1; j--) // reverse for boomerang
          {
            let t = PcCameraControl.flashStamp[i] + (j*1.5/7);
            f = await extractFrame(video, canvas, t);
            PcCaptureSession.addFrame(f, true);
          }

          PcCaptureSession.nextShot();
      }

      this.makeShootsFilters();
    }

    video.preload = true;
    video.src = videoFile;
  }

  makeShootsFilters()
  {
    var currentFormat = PcConfig.getMemory("currentFormat");

    PcConfig.framingAlert.splice(0, PcConfig.framingAlert.length);

    if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
    {
      console.log("make shoots no webcam");
      for (let i=0; i<PcConfig.screen.templateAndFilter.filters.length; i++)
      {
        console.log("make shoots no webcam " + i);
        PcCaptureSession.defaultFilter(PcConfig.screen.templateAndFilter.filters[i])
      }

      this.makeId();
    }
    else {

      if (currentFormat.autocrop)
      {
        let template = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout);
        let frameSize = {width:template.poses[0].width, height:template.poses[0].height}

        PcCaptureSession.applyFraming(()=>{
          for (let i=0; i<PcConfig.screen.templateAndFilter.filters.length; i++)
          {
            PcCaptureSession.filter(PcConfig.screen.templateAndFilter.filters[i])
          }

          if (PcConfig.getMemory("winner")==1) PcConfig.sounds.win.play();

          this.makeId();
        }, currentFormat.autocropSize, currentFormat.autocropPosition, frameSize, !currentFormat.autocropSoft);
      }
      else
      {
        for (let i=0; i<PcConfig.screen.templateAndFilter.filters.length; i++)
        {
          PcCaptureSession.filter(PcConfig.screen.templateAndFilter.filters[i])
        }

        if (PcConfig.getMemory("winner")==1) PcConfig.sounds.win.play();

        this.makeId();
      }
    }

  }

  makeId()
  {
    if (PcConfig.screen.format.idActive)
    {
      PcCaptureSession.createIdShot(()=>{
        this.props.goNextScreen();
      }, PcConfig.screen.format.idAutocropSize, PcConfig.screen.format.idAutocropPosition, {width:PcConfig.screen.format.idWidth, height:PcConfig.screen.format.idHeight});
    }
    else {
      this.props.goNextScreen();
    }
  }

  getWinner(fOnLoad = null)
  {
    if (!PcConfig.instantWin.active)
    {
      fOnLoad();
      return;
    }

    PcConfig.setMemory("winner", 0);

    $.ajax({
      url: PcConfig.host + 'scripts/get_winner.php?config=' + PcConfig.global.id,
      type: "GET",
      timeout:10000,
      success: (data)=>{
        var json = PcConfig.xmlToJson(PcConfig.XMLParser.parseFromString(data,"text/xml"));
        console.log(json);
        if (json.xml.id_lot._value!=0)
        {
          console.log("WINNER : lot " + json.xml.gain._value + " - id " + json.xml.id_lot._value);
          PcConfig.setMemory("winner", 1);
          PcConfig.setMemory("winnerLot", json.xml.gain._value);
          PcConfig.setMemory("winnerIdLot", json.xml.id_lot._value);
        }
        if (fOnLoad) fOnLoad();
      },
      error: (xhr, status, err)=>{
      }
    });
  }

  render()
  {
    if (!this.props.active) return null;

    this.getWinner(()=>{
      this.testShoots();
    });

    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.props.noAnimation?"":this.state.animationStyle
        },
      },

      e(PcLabel,
        {
          label:PcConfig.screen.loading.title,
          title:true,
          width:this.props.width,
          height:"15%",
          fontSize:PcConfig.scenario.fontSizeBig,
          color: PcConfig.scenario.fontColor,
          showDelay:"0,1s"
        }
      ),

      e(PcAnimationLoading,
        {
          x:(this.props.width-(400*PcConfig.renderSize))/2,
          y:(this.props.height-(400*PcConfig.renderSize))/2,
          color: PcConfig.scenario.fontColor,
          showDelay:"0.2s"
        }
      )

    );
  }
}
