class PcScenario extends React.Component
{
  constructor(props)
  {
    super(props);

    if (PcConfig.useElectron()) {
      this.fs = require('electron').remote.require("fs");
      this.process = require('electron').remote.require("child_process");
    }

    this.numScreens = 8;
    this.startScreen = 0;
    this.scenario = [];
    this.screen = [];
    this.sleepTimer;
    this.sleepTimerFreezed = false;
    this.sleepTimerTime = PcConfig.scenario.sleepTimer;

    var screenStates = [];
    for (let i=0; i<this.numScreens; i++) {screenStates[i] = false;}

    this.screenCamera = React.createRef();
    this.screenInfos = React.createRef();

    this.currentScreenIndex = 0;

    PcCameraControl.init();

    this.state = {
      screenActive:screenStates,
      cameraActive:false,
      screenBackground:PcConfig.screen.intro.background,
      screenForeground:"",
      cameraForeground:"",
      screenPreviewFile:"",
      screenCaptureTitle : "",
      displayMainLoader:"block",
    }

    this.buildScenario();
    this.resetSleepTimer();

    setTimeout(()=>{this.setState({displayMainLoader:"none"}); this.goScreen(this.startScreen)}, 2000);
  }



  buildScenario()
  {
      this.scenario[0] = {name:"intro", prev:0, next:1};
      this.scenario[1] = {name:"format", prev:0, next:2};
      this.scenario[2] = {name:"start", prev:1, next:3};
      this.scenario[3] = {name:"capture", prev:2, next:4};
      this.scenario[4] = {name:"loading", prev:3, next:5};
      this.scenario[5] = {name:"templateAndFilter", prev:1, next:6};
      this.scenario[6] = {name:"infos", prev:5, next:7};
      this.scenario[7] = {name:"end", prev:6, next:0};
  }

  resetSleepTimer()
  {
    if (this.sleepTimerFreezed) return;
    clearTimeout(this.sleepTimer);
    this.sleepTimer = setTimeout(()=>this.onSleepTimer(), this.sleepTimerTime);
  }

  stopSleepTimer()
  {
    clearTimeout(this.sleepTimer);
  }

  freezeSleepTimer()
  {
    clearTimeout(this.sleepTimer);
    this.sleepTimerFreezed = true;
  }

  unfreezeSleepTimer()
  {
    this.sleepTimerFreezed = false;
  }

  onSleepTimer()
  {
    if (this.state.screenActive[7])
    {
      window.location.reload();
    }
    else {
      this.goScreen(0);
    }
  }

  goPrevScreen(e, screenIndex)
  {
    var nextScreenIndex = this.scenario[screenIndex].prev;
    this.goScreen(nextScreenIndex, -1);
  }

  goNextScreen(e, screenIndex)
  {
    var nextScreenIndex = this.scenario[screenIndex].next;
    this.goScreen(nextScreenIndex, 1);
  }

  goScreen(nextScreenIndex, increment)
  {
    console.log("goscreen " + this.scenario[nextScreenIndex].name);

    if (PcConfig.useElectron()) PcConfig.electron.send('windowTop');

    this.currentScreenIndex = nextScreenIndex;

    var screenStates = [];
    var cameraActive = false;
    var screenBackground = "";
    var screenForeground = "";
    var cameraForeground = this.state.cameraForeground;
    var screenCaptureTitle = this.state.screenCaptureTitle;
    var endLabel = "";//PcConfig.screen.end.title;
    var endLoaded = false;
    var formatKeepState = false;
    var previewKeepState = false;
    var formKeepState = false;
    var langButtonVisible = false;

    for (let i=0; i<this.numScreens; i++) {screenStates[i] = false;}
    screenStates[nextScreenIndex] = true;

    switch (this.scenario[nextScreenIndex].name)
    {
      case "intro":
        screenBackground = PcConfig.screen.intro.background,
        formatKeepState = true;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.stopSleepTimer();
        cameraActive = false;
        PcCameraControl.stopLiveView();
        PcConfig.resetMemory();
        PcCaptureSession.reset();
        break;

      case "format":
        screenBackground = PcConfig.screen.format.background,
        formatKeepState = true;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        cameraActive = false;

        if (PcConfig.getMemory("currentFormat")=="")
        {
          if (PcConfig.screen.format.formats.length<=2)
          {
            PcConfig.setMemory("currentFormat", PcConfig.screen.format.formats[0]);
            PcConfig.setMemory("currentFormatIndex", 0);
          }
          else {
            let f = Math.ceil((PcConfig.screen.format.formats.length-1)/2);
            PcConfig.setMemory("currentFormat", PcConfig.screen.format.formats[f]);
            PcConfig.setMemory("currentFormatIndex", f);
          }
        }

        if (PcConfig.screen.format.formats.length<2)
        {
          increment==1 ? this.goNextScreen(null, nextScreenIndex) : this.goPrevScreen(null, nextScreenIndex);
          return;
        }

        PcCameraControl.stopLiveView();

        break;

      case "start":
        screenBackground = PcConfig.screen.start.background,
        cameraForeground =  "";
        formatKeepState = true;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.sleepTimerTime = PcConfig.scenario.sleepTimer;
        this.resetSleepTimer();
        cameraActive = true;
        PcCameraControl.startLiveView();
        PcConfig.setMemory('formShouldReset', true);
        PcCaptureSession.reset();
        PcCameraControl.deleteOldShoots();
        PcConfig.setMemory("currentSkinIndex", "");

        let fIndex = PcConfig.screen.templateAndFilter.defaultFilterIndex;
        if (fIndex>=PcConfig.screen.templateAndFilter.filters.length) fIndex = PcConfig.screen.templateAndFilter.filters.length-1;
        if (fIndex<0) fIndex = 0;
        PcConfig.setMemory("currentFilterIndex", fIndex);

        break;

      case "capture":
        screenBackground = PcConfig.screen.capture.background,
        cameraForeground =  "";
        formatKeepState = false;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = false;
        this.freezeSleepTimer();
        cameraActive = true;
        break;

      case "loading":
        screenBackground = PcConfig.screen.loading.background,
        formatKeepState = false;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = false;
        this.freezeSleepTimer();
        cameraActive = false;
        break;

      case "templateAndFilter":
        if (PcConfig.getMemory("winner")==1)
        {
          screenBackground = PcConfig.instantWin.background;
        }
        else {
          if (PcConfig.framingAlert.length>0)
          {
            screenBackground = PcConfig.screen.templateAndFilter.framingBackground;
          }
          else
          {
            screenBackground = PcConfig.screen.templateAndFilter.background;
          }
        }
        formatKeepState = false;
        previewKeepState = true;
        formKeepState = true;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.sleepTimerTime = PcConfig.scenario.sleepTimer;
        this.resetSleepTimer();
        PcCameraControl.startLiveView();

        if (PcConfig.getMemory("currentSkinIndex")=="")
        {
          if (PcConfig.getMemory("winner")==1)
          {
            PcConfig.setMemory("currentSkin", PcConfig.instantWin.lots[PcConfig.getMemory("winnerLot")][PcConfig.getMemory("currentFormatIndex")][0]);
            PcConfig.setMemory("currentSkinIndex", 0);
          }
          else if (PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")].length<=2)
          {
            PcConfig.setMemory("currentSkin", PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")][0]);
            PcConfig.setMemory("currentSkinIndex", 0);
          }
          else {
            let s = Math.ceil((PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")].length-1)/2);
            PcConfig.setMemory("currentSkin", PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")][s]);
            PcConfig.setMemory("currentSkinIndex", s);
          }
        }

        PcConfig.setMemory("currentFilter", PcConfig.screen.templateAndFilter.filters[PcConfig.getMemory("currentFilterIndex")]);
        //PcConfig.setMemory("currentFilterIndex", 0);

        break;

      case "infos":

        if (PcConfig.screen.infos.questions.length<1)
        {
          increment==1 ? this.goNextScreen(null, nextScreenIndex) : this.goPrevScreen(null, nextScreenIndex);
          return;
        }

        screenBackground = PcConfig.screen.infos.background
        formatKeepState = false;
        previewKeepState = true;
        formKeepState = true;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.sleepTimerTime = PcConfig.scenario.sleepTimer;
        this.resetSleepTimer();
        PcCameraControl.stopLiveView();
        if (PcConfig.screen.infos.listingFile!="") PcConfig.setMemory("formShouldReset", true);
        break;

      case "end":
        screenBackground = PcConfig.screen.end.background
        formatKeepState = false;
        previewKeepState = false;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.sleepTimerTime = PcConfig.scenario.sleepTimer;
        this.resetSleepTimer();
        endLoaded = false;
        setTimeout(()=>this.saveAll(()=>{
            this.setState({endLoaded:true})
          }), 100);
        break;
      default:
    };

    if (PcConfig.scenario.languages.length<=1) langButtonVisible = false;

    this.setState(
      {
        screenActive : screenStates,
        cameraActive : cameraActive,
        screenBackground : screenBackground,
        cameraForeground : cameraForeground,
        screenPreviewFile: PcConfig.getMemory('recordedVideoFile'),
        formatKeepState:formatKeepState,
        previewKeepState:previewKeepState,
        formKeepState:formKeepState,
        langButtonVisible:langButtonVisible,
        endLabel:endLabel,
        endLoaded:endLoaded,
      });

  }

  saveAll(onSuccess)
  {
    var format = PcConfig.getMemory("currentFormat");
    var skin = PcConfig.getMemory("currentSkin");
    var filter = PcConfig.getMemory("currentFilter");
    var shots;

    if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
    {
      shots =PcCaptureSession.defaultShot;
    }
    else {
      shots =PcCaptureSession.shot;
    }

    var photoData = PcTemplates.createPhotoData(format, skin, filter, shots);

    if (format.animation=="gif4")
    {
      PcTemplates.createGif4Data(format, skin, filter, shots, (gifData)=>{
        PcConfig.callSaveContent(photoData, true, (id_photo)=>{
          if (format.printing) this.printAll(id_photo);
          PcConfig.callSaveGif(gifData, id_photo, onSuccess);
        });
      });
    }
    else if (format.animation=="boomerang")
    {
      PcTemplates.createBoomerangData(format, skin, filter, shots, (gifData)=>{
        PcConfig.callSaveContent(photoData, true, (id_photo)=>{
          if (format.printing) this.printAll(id_photo);
          PcConfig.callSaveGif(gifData, id_photo, onSuccess);
        });
      });
    }
    else
    {
      PcConfig.callSaveContent(photoData, PcConfig.screen.format.idActive, (id_photo)=>{
        if (format.printing) this.printAll(id_photo);

        if (PcConfig.screen.format.idActive)
        {
          var canvas = PcTemplates.getZoomCropCanvas(PcCaptureSession.idFrame.canvas, PcConfig.screen.format.idWidth,PcConfig.screen.format.idHeight, false);
          var idData = canvas.toDataURL('image/jpeg', 0.9).replace(/^data:image\/\w+;base64,/, "");
          PcConfig.callSaveGif(idData, id_photo, onSuccess, true);
        }

        setTimeout(()=>{if (onSuccess) onSuccess();}, 1000);
      });
    }
  }

  printAll(id_photo)
  {
    var skin = PcConfig.getMemory("currentSkin");
    var skinColor = skin.color!="" ? skin.color : "#FFFFFF";

    $.ajax
  	({
  		url: 'http://localhost/scripts/getmediatoprint.php?id=' + id_photo + "&bgcolor=" + skinColor,
  		success: (response) => {
  			var r = $.parseJSON(response);
  			var file = "c:/www/photos/" + r.file;
  			var bgcolor = skinColor.replace("#", "");
  			var prints = PcConfig.getMemory("prints")!="" ? PcConfig.getMemory("prints") : 1;
  			var crop = r.crop;
  			var cmd = "c:/www/printer/autoprint.exe \"" + file + "\" " + bgcolor + " " + prints + " " + crop;
        //console.log(cmd);
        if (PcConfig.useElectron())
        {
          let process = require('electron').remote.require("child_process");
          process.exec(cmd);
        }
  		}
  	});
  }

  changeLanguage(l)
  {
    if (PcConfig.currentLanguage==l) return;

    PcConfig.currentLanguage=l;

    let screenStates = [];
    for (let i=0; i<this.numScreens; i++) {screenStates[i] = false;}
    this.setState({screenActive : screenStates});

    setTimeout(()=>{
      screenStates[this.currentScreenIndex] = true;
      this.setState({screenActive : screenStates})
    }, 100);
  }

  render()
  {
    let globalWidth=0;
    let globalHeight=0;

    if (this.props.mode=="photokiosk")
    {
      globalWidth = 1080 * this.props.renderSize;
      globalHeight = 1920 * this.props.renderSize;
    }
    else if (this.props.mode=="photobooth")
    {
      if (this.props.device=="surface")
      {
        globalWidth = 1920 * this.props.renderSize;
        globalHeight = globalWidth * 2 / 3;
      }
      else
      {
        globalWidth = 1920 * this.props.renderSize;
        globalHeight = 1080 * this.props.renderSize;
      }
    }
    else if (this.props.mode=="cyclomaton")
    {
      globalWidth = 1920 * this.props.renderSize;
      globalHeight = 1080 * this.props.renderSize;
    }
    else
    {
      globalWidth = 1080 * this.props.renderSize;
      globalHeight = 1920 * this.props.renderSize;
    }

    return e("div",
      {
        className: this.props.view=="preview" ? "pcscenariopreview" :  "pcscenario",
        style: {
          width:globalWidth,
          height:globalHeight,
          backgroundColor:PcConfig.scenario.backgroundColor,
          border: this.props.view=="preview" ? parseInt(4*this.props.renderSize)+"px solid #888888" : "",
        },
        onClick:()=>{this.resetSleepTimer(PcConfig.scenario.sleepTimer)},
        onMouseMove:()=>this.resetSleepTimer(PcConfig.scenario.sleepTimer),
        onTouchEnd:()=>this.resetSleepTimer(PcConfig.scenario.sleepTimer),
      },
      e(PcMedia,
        {
          width:globalWidth,
          height:globalHeight,
          content:PcConfig.scenario.background
        }
      ),
      e(PcMedia,
        {
          width:globalWidth,
          height:globalHeight,
          content:this.state.screenBackground
        }
      ),
      e(PcScreenIntro,
        {
          active:this.state.screenActive[0],
          goNextScreen:(e)=>this.goNextScreen(e, 0),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenFormat,
        {
          active:this.state.screenActive[1],
          goPrevScreen:(e)=>this.goPrevScreen(e, 1),
          goNextScreen:(e)=>this.goNextScreen(e, 1),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenStart,
        {
          active:this.state.screenActive[2],
          goPrevScreen:(e)=>this.goPrevScreen(e, 2),
          goNextScreen:(e)=>this.goNextScreen(e, 2),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenCapture,
        {
          active:this.state.screenActive[3],
          startRecording:()=>this.screenCamera.current.startRecording(),
          goNextScreen:(e)=>this.goNextScreen(e, 3),
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcScreenLoading,
        {
          active:this.state.screenActive[4],
          goNextScreen:(e)=>this.goNextScreen(e, 4),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenTemplateAndFilter,
        {
          active:this.state.screenActive[5],
          goPrevScreen:(e)=>this.goPrevScreen(e, 5),
          goNextScreen:(e)=>this.goNextScreen(e, 5),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenInfos,
        {
          active:this.state.screenActive[6],
          keepState:this.state.formKeepState,
          ref:this.screenInfos,
          goPrevScreen:(e)=>this.goPrevScreen(e, 6),
          goNextScreen:(e)=>this.goNextScreen(e, 6),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenEnd,
        {
          active:this.state.screenActive[7],
          goPrevScreen:(e)=>this.goPrevScreen(e, 7),
          goNextScreen:(e)=>this.goNextScreen(e, 7),
          width:globalWidth,
          height:globalHeight,
          endLoaded:this.state.endLoaded,
        }
      ),
      e(PcLanguages,
        {
          visible:this.state.langButtonVisible,
          changeLanguage:(l)=>{this.changeLanguage(l)},
          x:0,
          y:0,
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcConnection,
        {
          visible:false,
          x:0,
          y:0,
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcScreenMainLoading,
        {
          display:this.state.displayMainLoader,
          width:globalWidth,
          height:globalHeight,
          content:PcConfig.scenario.background
        }
      )
    );
  }
}
