class PcCaptureSession
{
  static initialize(fOnLoad=null)
  {
    PcCaptureSession.shot = [];

    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;

    PcCaptureSession.idShot = null;
    PcCaptureSession.idFrame = null;

    PcCaptureSession.defaultShot = [];
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;

    PcCaptureSession.resources = [];

    PcCaptureSession.intializeResources(()=>{
      PcCaptureSession.loadDefaults();
      if (fOnLoad) fOnLoad();
    });

    PcCaptureSession.framing = new PcFraming();
    PcCaptureSession.framing.initialize();
  }

  static intializeResources(fOnLoad=null)
  {
    let urlsToLoad = [];
    let k=0;

    let testLoading = (url)=>{
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].url==url) urlsToLoad[i].loaded = true;}
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].loaded==false) return;}
        if (fOnLoad) fOnLoad();
    }

    urlsToLoad.push({name:"filter_general_vintagecover", url:PcConfig.host + "ressources/filter_general_vintagecover.png", loaded:false});
    urlsToLoad.push({name:"filter_general_whiteglow", url:PcConfig.host + "ressources/filter_general_whiteglow.png", loaded:false});
    urlsToLoad.push({name:"filter_general_yellowglow", url:PcConfig.host + "ressources/filter_general_yellowglow.png", loaded:false});

    if (PcConfig.scenario.mode=="photobooth")
    {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/booth_frame1.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/booth_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/booth_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/booth_frame4.jpg?x="+Math.random(), loaded:false});
    }
    else {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/kiosk_frame1.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/kiosk_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/kiosk_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/kiosk_frame4.jpg?x="+Math.random(), loaded:false});
    }

    for (let i=0; i<urlsToLoad.length; i++) {PcCaptureSession.loadResource(urlsToLoad[i].name, urlsToLoad[i].url, testLoading);}

    if (urlsToLoad.length==0) {if (fOnLoad) fOnLoad();}
  }

  static loadDefaults()
  {
    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f1"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f2"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f3"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f4"), true);

    setTimeout(()=>this.defaultFilter(PcConfig.screen.format.filter), 2000);
  }

  static nextShot()
  {
    if (PcCaptureSession.shot[PcCaptureSession.currentShot])
    {
        PcCaptureSession.currentShot++;
    }
  }

  static nextDefaultShot()
  {
    if (PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot])
    {
        PcCaptureSession.defaultCurrentShot++;
    }
  }

  static addFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.shot[PcCaptureSession.currentShot])
    {
      PcCaptureSession.shot[PcCaptureSession.currentShot] = new PcCaptureSessionShot();
      PcCaptureSession.numShots++;
    }

    PcCaptureSession.shot[PcCaptureSession.currentShot].addFrame(data, dataIsCanvas);

    if (PcCaptureSession.currentShot == 0)
    {
      if (PcCaptureSession.shot[0].frame.length == 1)
      {
        this.idFrame = new PcCaptureSessionFrame(data, dataIsCanvas);
      }
    }

  }

  static addDefaultFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot])
    {
      PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot] = new PcCaptureSessionShot();
      PcCaptureSession.defaultNumShots++;
    }

    PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot].addFrame(data, dataIsCanvas);
  }

  static async applyFraming(onApplyFraming, faceRatio, centerRatio, frameSize, error)
  {
    for (let i=0; i<PcCaptureSession.shot.length; i++)
    {
      await PcCaptureSession.shot[i].framing(PcCaptureSession.framing, faceRatio, centerRatio, frameSize, error);
    }

    onApplyFraming();
  }

  static async createIdShot(onApplyFraming, faceRatio, centerRatio, frameSize)
  {
    //PcCaptureSession.idShot = new PcCaptureSessionShot();
    //PcCaptureSession.idShot.addFrame(PcCaptureSession.idFrame.canvas, true);

    await PcCaptureSession.idFrame.framing(PcCaptureSession.framing, faceRatio, centerRatio, frameSize, true);

    onApplyFraming();
  }

  static filter(filter)
  {
    PcCaptureSession.shot.forEach((shot, index) => shot.filter(filter, index));
  }

  static defaultFilter(filter)
  {
    PcCaptureSession.defaultShot.forEach((shot, index) => shot.filter(filter, index));
  }

  static reset()
  {
    PcCaptureSession.shot.forEach(shot => shot.delete());
    PcCaptureSession.shot.splice(0);
    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;
  }

  static resetDefault()
  {
    PcCaptureSession.defaultShot.forEach(shot => shot.delete());
    PcCaptureSession.defaultShot.splice(0);
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;
  }

  static loadResource(name, imgUrl, fOnLoad=null)
  {
    if (imgUrl=="") return;

    PcCaptureSession.resources[name] = new Image();

    if (fOnLoad)
    {
      PcCaptureSession.resources[name].onload = ()=>{fOnLoad(imgUrl)};
      PcCaptureSession.resources[name].onerror = ()=>{fOnLoad(imgUrl)};
    }

    PcCaptureSession.resources[name].src = imgUrl;
  }

  static getResource(name)
  {
    let img = PcCaptureSession.resources[name];
    let canvas = document.createElement('canvas');
    canvas.width = img.naturalWidth;
    canvas.height = img.naturalHeight;
    canvas.getContext('2d').drawImage(img, 0, 0);
    return canvas;
  }

  static cloneCanvas(oldCanvas)
  {
    var newCanvas = document.createElement('canvas');
    var context = newCanvas.getContext('2d');

    newCanvas.width = oldCanvas.width;
    newCanvas.height = oldCanvas.height;

    context.drawImage(oldCanvas, 0, 0);

    return newCanvas;
  }

}
