const MODEL_URL = 'js/lib/face-api/weights';

var sleep = ms => new Promise(resolve => setTimeout(resolve, ms));

faceapi.env.monkeyPatch({
    Canvas: HTMLCanvasElement,
    createCanvasElement: () => document.createElement('canvas')
})

class PcFraming
{
  constructor()
  {
    this.faceRatio = 2.4;
    this.finalRatio = 1;
    this.resultWidth = 1080;
    this.resultHeight = 1080;

    this.originalImage = null;

    this.originalCanvasSampled = null;
    this.finalCanvas = null;
    this.sampleRatio = 8;

    this.faceWidth = 0;
    this.faceHeight = 0;
    this.faceRect = null;
    this.mouthX = 0;
    this.mouthY = 0;

    this.fileSaved = false;
    this.outputFile = "";

    this.onInitialized = null;
    this.onRunned = null;
  }

  async initialize(onInitialized)
  {
    faceapi.tf.setBackend('cpu');

    //await faceapi.loadSsdMobilenetv1Model(MODEL_URL);
    await faceapi.loadTinyFaceDetectorModel(MODEL_URL);
    await faceapi.loadFaceLandmarkModel(MODEL_URL);

    this.onInitialized = onInitialized;
    if (this.onInitialized) this.onInitialized();
  }

  async apply(canvas)
  {
    this.sampleRatio = canvas.width/600;

    this.createOriginalCanvasSampled(canvas);

    this.originalImage = canvas;

    await this.detectFace();
    this.computeResult();

    this.originalCanvasSampled = null;

    return this.finalCanvas;
  }

  createOriginalCanvasSampled(canvas)
  {
    this.originalCanvasSampled = document.createElement("canvas");
    this.originalCanvasSampled.width = canvas.width / this.sampleRatio;
    this.originalCanvasSampled.height = canvas.height / this.sampleRatio;
    var ctx = this.originalCanvasSampled.getContext('2d');
    ctx.drawImage(canvas, 0, 0, this.originalCanvasSampled.width, this.originalCanvasSampled.height);
  }

  async detectFace()
  {
    //const options = new faceapi.SsdMobilenetv1Options({ minConfidence: 0.1 })
    const options = new faceapi.TinyFaceDetectorOptions({ minConfidence: 0.1 })
    const results = await faceapi.detectSingleFace(this.originalCanvasSampled, options).withFaceLandmarks();

    if (!results)
    {
      //this.faceWidth = this.originalImage.width * this.sampleRatio;
      this.faceWidth = 0;
      this.mouthX = 0;
      this.mouthY = 0;
      this.faceRect = {width:this.originalImage.width, height:this.originalImage.height, x:0, y:0};
      console.warn("No face detected");
      return;
    }

    const mouth = results.landmarks.getMouth();
    const nose = results.landmarks.getNose();
    const leftEye = results.landmarks.getLeftEye();
    const rightEye = results.landmarks.getRightEye();
    const leftEyeBrow = results.landmarks.getRightEyeBrow();
    const rightEyeBrow = results.landmarks.getRightEyeBrow();
    const jawOutline = results.landmarks.getJawOutline();

    var minMouthX = Infinity;
    var minMouthY = Infinity;
    var maxMouthX = 0;
    var maxMouthY = 0;

    var zone = nose.concat(mouth, leftEye, rightEye, leftEyeBrow, rightEyeBrow, jawOutline);

    zone.forEach((e)=> {
      if (e.x<minMouthX) minMouthX = e.x;
      if (e.x>maxMouthX) maxMouthX = e.x;
      if (e.y<minMouthY) minMouthY = e.y;
      if (e.y>maxMouthY) maxMouthY = e.y;
    })

    this.faceRect = {};
    this.faceRect.width = (maxMouthX-minMouthX) * this.sampleRatio;
    this.faceRect.height = (maxMouthY-minMouthY) * this.sampleRatio;
    this.faceRect.x = minMouthX * this.sampleRatio;
    this.faceRect.y = minMouthY * this.sampleRatio;

    this.mouthX = this.faceRect.x + this.faceRect.width/2;
    this.mouthY = this.faceRect.y + this.faceRect.height/3;

    this.faceWidth = this.faceRect.width;
    this.faceHeight = this.faceRect.height;
  }

  computeResult()
  {
    var finalWidth = this.faceWidth * this.faceRatio;
    var finalHeight = finalWidth * this.finalRatio;
    var finalX = this.mouthX - 0.5*finalWidth;
    var finalY = this.mouthY - 0.5*finalHeight;


    if (finalWidth==0)
    {
      finalHeight = this.originalImage.height;
      finalWidth = finalHeight / this.finalRatio;
      finalX = (this.originalImage.width - finalWidth)/2;
      finalY = (this.originalImage.height - finalHeight)/2;
      this.mouthX = this.originalImage.width/2;
      this.mouthY = this.originalImage.height/2;
    }

    /*
    this.originalCanvasSampled.getContext('2d').beginPath();
    this.originalCanvasSampled.getContext('2d').lineWidth = "2";
    this.originalCanvasSampled.getContext('2d').strokeStyle = "green";
    this.originalCanvasSampled.getContext('2d').rect(finalX/this.sampleRatio, finalY/this.sampleRatio, finalWidth/this.sampleRatio, finalHeight/this.sampleRatio);
    this.originalCanvasSampled.getContext('2d').stroke();
    */

    //test bounds and max sizes
    /*
    if ((this.mouthY + 0.5*finalHeight) > this.originalImage.height)
    {
      let dif = ((this.mouthY + 0.5*finalHeight) - this.originalImage.height);
      finalHeight = finalHeight - 2*dif;
      finalWidth = finalHeight / this.finalRatio;
      finalY = finalY + dif;
      finalX = finalX + dif/this.finalRatio;
    }

    if ((this.mouthX + 0.5*finalWidth) > this.originalImage.width)
    {
      let dif = ((this.mouthX + 0.5*finalWidth) - this.originalImage.width);
      finalWidth = finalWidth - 2*dif;
      finalHeight = finalWidth * this.finalRatio;
      finalX = finalX + dif;
      finalY = finalY + dif*this.finalRatio;
    }

    if ((this.mouthY - 0.5*finalHeight) < 0)
    {
      let dif = 0.5*finalHeight - this.mouthY;
      finalHeight = finalHeight - 2*dif;
      finalWidth = finalHeight / this.finalRatio;
      finalY = finalY + dif;
      finalX = finalX + dif/this.finalRatio;
    }

    if ((this.mouthX - 0.5*finalWidth) < 0)
    {
      let dif = 0.5*finalWidth - this.mouthX;
      finalWidth = finalWidth - 2*dif;
      finalHeight = finalWidth * this.finalRatio;
      finalX = finalX + dif;
      finalY = finalY + dif*this.finalRatio;
    }
    */

    if (finalHeight>this.originalImage.height)
    {
      let dif = finalHeight - this.originalImage.height;
      finalHeight = this.originalImage.height;
      finalWidth = finalHeight / this.finalRatio;
      finalY = 0;
      finalX = finalX + 0.5*dif/this.finalRatio;
      this.mouthY = finalHeight/2;
      console.log("error height");
    }

    if (finalWidth>this.originalImage.width)
    {
      let dif = finalWidth - this.originalImage.width;
      finalWidth = this.originalImage.width;
      finalHeight = finalWidth * this.finalRatio;
      finalX = 0;
      finalY = finalY + 0.5*dif*this.finalRatio;
      this.mouthX = finalWidth/2;
      console.log("error width");
    }

    if ((this.mouthY + 0.5*finalHeight) > this.originalImage.height)
    {
      let dif = ((this.mouthY + 0.5*finalHeight) - this.originalImage.height);
      finalY = finalY - dif;
      console.log("error down");
    }

    if ((this.mouthX + 0.5*finalWidth) > this.originalImage.width)
    {
      let dif = ((this.mouthX + 0.5*finalWidth) - this.originalImage.width);
      finalX = finalX - dif;
      console.log("error right");
    }

    if ((this.mouthY - 0.5*finalHeight) < 0)
    {
      let dif = 0.5*finalHeight - this.mouthY;
      finalY = finalY + dif;
      console.log("error up");
    }

    if ((this.mouthX - 0.5*finalWidth) < 0)
    {
      let dif = 0.5*finalWidth - this.mouthX;
      finalX = finalX + dif;
      console.log("error left");
    }


    var originalCanvas = document.createElement("canvas");
    originalCanvas.width = this.originalImage.width;
    originalCanvas.height = this.originalImage.height;
    var ctx = originalCanvas.getContext('2d');
    ctx.drawImage(this.originalImage, 0, 0, originalCanvas.width, originalCanvas.height);

    //facerect
    ctx.beginPath();
    ctx.lineWidth = "2";
    ctx.strokeStyle = "blue";
    ctx.rect(this.faceRect.x, this.faceRect.y, this.faceRect.width, this.faceRect.height);
    ctx.stroke();

    //center
    ctx.beginPath();
    ctx.lineWidth = "4";
    ctx.strokeStyle = "green";
    ctx.rect(this.mouthX-2, this.mouthY-2, 4, 4);
    ctx.stroke();

    this.finalCanvas = document.createElement("canvas");
    this.finalCanvas.width = this.resultWidth;
    this.finalCanvas.height = this.resultHeight;

    this.finalCanvas.getContext('2d').fillStyle = "black";
    this.finalCanvas.getContext('2d').fillRect(0, 0, this.finalCanvas.width, this.finalCanvas.height);
    this.finalCanvas.getContext('2d').drawImage(this.originalImage, finalX, finalY, finalWidth, finalHeight, 0, 0, this.resultWidth, this.resultHeight);

    PcFraming.brightness(this.finalCanvas, 6);
    PcFraming.contrast(this.finalCanvas, 12);
    //this.finalCanvas.getContext('2d').drawImage(originalCanvas, finalX, finalY, finalWidth, finalHeight, 0, 0, this.resultWidth, this.resultHeight);
  }

  static brightness(canvas, brightness)
  {
    var context = canvas.getContext('2d');
    var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
    var data = imgData.data;

    for (var i = 0; i < data.length; i+= 4)
    {
      data[i] += 255 * (brightness / 100);
      data[i+1] += 255 * (brightness / 100);
      data[i+2] += 255 * (brightness / 100);
    }

    context.putImageData(imgData, 0, 0);
  }

  static contrast(canvas, contrast)
  {
    var context = canvas.getContext('2d');
    var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
    var data = imgData.data;
    var factor = (259 * (contrast + 255)) / (255 * (259 - contrast));

    for(var i=0;i<data.length;i+=4)
    {
        data[i] = factor * (data[i] - 128) + 128;
        data[i+1] = factor * (data[i+1] - 128) + 128;
        data[i+2] = factor * (data[i+2] - 128) + 128;
    }

    context.putImageData(imgData, 0, 0);
  }
}
