class PcButtonPreview extends React.Component
{
  static get test() {return "test"}

  constructor(props)
  {
    super(props);
    this.state =
      {
        width:props.width,
        padHorizontal:0,
        padVertical:0
      }

    this.buttonBackground = React.createRef();
    this.buttonCanvas = React.createRef();
    this.buttonContainer = React.createRef();
    this.buttonFlash = React.createRef();

    this.animationInterval = null;
  }

  startAnimation()
  {
    this.updateCanvas();
    this.doFlash();

    if (this.props.kind=="gif")
    {
      this.animationInterval = setInterval(()=>this.updateCanvas(), 60);
    }
    else {
      this.animationInterval = setInterval(()=>this.updateCanvas(), 60);
    }
  }

  stopAnimation()
  {
    clearInterval(this.animationInterval);
  }

  doFlash()
  {
    setTimeout(()=>{this.buttonFlash.current.style.transition=""}, 1);
    setTimeout(()=>{this.buttonFlash.current.style.opacity=0.5}, 20);
    setTimeout(()=>{this.buttonFlash.current.style.transition="all 0.3s ease-out"}, 40);
    setTimeout(()=>{this.buttonFlash.current.style.opacity=0}, 60);
  }

  updateCanvas()
  {
    if (this.buttonCanvas.current)
    {
      let canvas = PcCameraControl.getCurrentFrame(this.props.width, this.props.height);

      this.buttonCanvas.current.width = this.props.width;
      this.buttonCanvas.current.height = this.props.height;
      this.buttonCanvas.current.getContext('2d').drawImage(canvas, 0, 0);
      canvas = null;
    }

    //if (this.props.kind=="photo") this.doFlash();
  }

  static predictSize(label)
  {
    var e = document.createElement("canvas");
    var ctx = e.getContext("2d");
    ctx.font = PcConfig.scenario.fontSize + " " + PcConfig.scenario.fontFamily;
    var t= ctx.measureText(label);
    return t.width;
  }

  getWidth()
  {
    return this.buttonBackground.current.getBoundingClientRect().width;
  }

  setWidth(w)
  {
    this.setState({width: w-(2*this.state.padHorizontal*PcConfig.renderSize)-2*PcConfig.scenario.buttonsBorderWidth});
  }

  onClick(e)
  {
    if (e.type === 'touchend')
    {
      //e.stopPropagation();
      e.preventDefault();
      if (e.targetTouches.length>0) return;
    }

    if (this.props.sound) this.props.sound.play();
    if (this.props.onClick) this.props.onClick();
  }

  render()
  {
    if (this.props.active)
    {
      this.startAnimation();
    }else {
      this.stopAnimation();
    }

    var borderColor = PcConfig.scenario.standardButtonBorderColor;
    var backgroundColor = PcConfig.scenario.standardButtonBackgroundColor;
    var textColor = PcConfig.scenario.standardButtonTextColor;
    var borderWidth = PcConfig.scenario.buttonsBorderWidth;
    var shadow = 5*PcConfig.renderSize;

    if (PcConfig.scenario.buttonsStyle=="rounded")
    {
      var borderRadius = 12 * PcConfig.renderSize;
    }
    else
    {
      var borderRadius = 0;
    }

    borderRadius = 0;
    borderWidth = 0;


    return e('div',
        {
          className:"pcbuttoncontainer",
          ref:this.buttonContainer,
          style:
          {
            opacity:0,
            animation:PcConfig.scenario.elementsAnimationStyle,
            animationDelay:this.props.showDelay,
            left: this.props.x,
            top:this.props.y,
            marginLeft: this.props.spacingH,
            marginRight: this.props.spacingH,
            marginTop: this.props.spacingV,
            marginBottom: this.props.spacingV
          }
        },
        e('div',
          {
            className:"pcbutton",
            ref:this.buttonBackground,
            style:
              {
                position:"relative",
                width:this.props.width,
                height:this.props.height,
                borderStyle:"solid",
                borderWidth:borderWidth,
                borderColor:borderColor,
                backgroundColor:"#000000",
                borderRadius:borderRadius,
                boxShadow: shadow + "px " + shadow + "px " + shadow + "px 0px rgba(0, 0, 0, 0.5)",
                paddingTop:this.state.padVertical*PcConfig.renderSize,
                paddingBottom:this.state.padVertical*PcConfig.renderSize,
                paddingLeft:this.state.padHorizontal*PcConfig.renderSize,
                paddingRight:this.state.padHorizontal*PcConfig.renderSize,
                overflow:"hidden",
                fontFamily:PcConfig.scenario.fontFamily,
                fontSize:PcConfig.scenario.fontSize,
                color:textColor,
                verticalAlign: "middle",
                textAlign: "center"
              },
              onClick:(e)=>{this.onClick(e)},
              onTouchEnd:(e)=>{this.onClick(e)},
          },
          e('img',
            {
              src:this.props.foreground,
              style:
              {
                position:"absolute",
                width:this.props.width,
                height:this.props.height,
                top:0,
                left:0,
                zIndex:200,
              }
            }
          ),
          e('div',
            {
              ref:this.buttonFlash,
              style:
              {
                position:"absolute",
                width:this.props.width,
                height:this.props.height,
                backgroundColor:"#ffffff",
                opacity:1,
                transition:"all 0.5s ease-out",
                zIndex:300,
              }
            }
          ),
          e('canvas',
            {
              ref:this.buttonCanvas,
              style:
              {
                position:"absolute",
                left:0,
                top:0,
                width:this.props.width,
                height:this.props.height,
                zIndex:10,
                transform: "scaleX(-1)",
              }
            }
          )
        ),
        e("div", {style:{width:"100%", height:20*PcConfig.renderSize}}),
        e(PcLabel,
          {
            label:this.props.label,
            width:this.props.width,
            height:this.props.height*(0.1),
            fontSize:PcConfig.scenario.fontSize,
            color: PcConfig.scenario.textColor,
            showDelay:"0,1s"
          }
        )
    );
  }
}
