class PcFormText extends PcFormElement
{
  constructor(props)
  {
    super(props);

    this.emailInput = React.createRef();
    this.okButton = React.createRef();

    this.state = {okButtonVisible:false}
  }

  setFocus()
  {
    ReactDOM.findDOMNode(this.emailInput.current).focus();
  }

  onInputChange()
  {
    this.props.answered(this.emailInput.current.value);
  }

  reset()
  {
    this.emailInput.current.value = "";
  }

  onKeyPressed(key)
  {
    let cpos = this.emailInput.current.selectionStart;
    let s1 = this.emailInput.current.value.substring(0,cpos);
    let s2 = this.emailInput.current.value.substring(cpos,this.emailInput.current.value.length);
    this.emailInput.current.setSelectionRange(cpos, cpos);

    if (key=="⌫")
    {
      s1 = s1.substring(0, s1.length-1);
      this.emailInput.current.value = s1 + s2;
      this.emailInput.current.focus();
      if (cpos==0)
      {
        this.emailInput.current.setSelectionRange(0, 0);
      } else {
        this.emailInput.current.setSelectionRange(cpos-1, cpos-1);
      }

    }
    else if (key=="←")
    {
      this.emailInput.current.focus();
      if (cpos>0)
      {
        this.emailInput.current.setSelectionRange(cpos-1, cpos-1);
      }
    }
    else if (key=="→")
    {
      this.emailInput.current.focus();
      this.emailInput.current.setSelectionRange(cpos+1, cpos+1);
    }
    else
    {
      if (this.props.max!="")
      {
        if (this.emailInput.current.value.length>=this.props.max) key="";
      }

      this.emailInput.current.value = s1 + key + s2;
      this.emailInput.current.focus();
      this.emailInput.current.setSelectionRange(cpos+key.length, cpos+key.length);
    }

    this.onInputChange();

    this.validate()=="" ? this.setState({okButtonVisible:true}) : this.setState({okButtonVisible:false});
  }

  setValue(v)
  {
    this.emailInput.current.value = v;
  }

  validate()
  {
    let test = this.emailInput.current.value.trim();
    let min = this.props.min!="" ? Number(this.props.min) : 0;
    let max = this.props.max!="" ? Number(this.props.max) : 999999999999;
    let num = test.length;

    if (test.length>0)
    {
        if (min<=num && num<=max) return "";
    }
    else {
      if (!this.props.required) return "";
    }

    return this.props.errorMessage!="" ? this.props.errorMessage : " ";
  }

  onClickOkButton()
  {
    setTimeout(()=>this.props.okClick(), 10);
  }

  render()
  {
    const okButton = e(PcFormOkButton,
        {
          key:"okbutton",
          ref:this.okButton,
          onClick:()=>this.onClickOkButton(),
          active:this.props.active,
          label:"",
          sound:PcConfig.sounds.keyboard,
          position:"relative",
          visibility:this.state.okButtonVisible,
          fontSize:PcConfig.scenario.fontSize,
          color: PcConfig.scenario.fontColor,
          spacingHLeft:5*PcConfig.renderSize,
          spacingHRight:5*PcConfig.renderSize,
          spacingV:5*PcConfig.renderSize,
          halign:"left",
          showDelay:"0,1s"
        }
      );

    return e('div',
      {
        className:"pctextcontainer",
        onClick:this.props.onClick,
        style:
        {
          textAlign:this.props.centered ? "center" : "left",
          transition:"all 0.25s ease-out",
        }
      },
        e(PcLabel,
          {
            label:this.props.label,
            position:"relative",
            width:this.props.width,
            height:this.props.height,
            halign : this.props.centered ? "center" : "left",
            fontSize:PcConfig.scenario.fontSize,
            color: PcConfig.scenario.fontColor,
            showDelay:"0,1s"
          }
        ),
        e('div', {style:{width:"100%", height:10*PcConfig.renderSize}}),
        e("input",
          {
            ref:this.emailInput,
            onChange:()=>this.onInputChange(),
            onInput:()=>this.onInputChange(),
            spellCheck:"false",
            style:
              {
                width:"98%",
                height:60*PcConfig.renderSize,
                fontFamily:PcConfig.scenario.fontFamily,
                fontSize:PcConfig.scenario.fontSize,
                color: PcConfig.scenario.fontColor,
                borderStyle:"solid",
                borderWidth:PcConfig.scenario.buttonsBorderWidth/2,
                borderColor:PcConfig.scenario.fontColor,
                borderRadius:12*PcConfig.renderSize,
                paddingLeft:10*PcConfig.renderSize,
                paddingRight:5*PcConfig.renderSize,
                textAlign:this.props.centered ? "center" : "left",
                background: "transparent",
                outline: "none",
              }
          }
        ),
        e('div', {style:{width:"100%", height:1*PcConfig.renderSize}}),
        e("div",
          {
            className:"pcok",
            style:
            {
              position:"relative",
              display:"inline-flex",
              justifyContent: this.props.centered ? "center" : "flex-start",
              flexWrap: "wrap",
              alignContent:"left",
              justifyContent:"left",
              animation:PcConfig.scenario.elementsAnimationStyle,
              animationDelay:this.props.showDelay,
            }
          },
          okButton
        ),
        e(PcLabel,
          {
            ref:this.mention,
            label:this.props.mention,
            position:"relative",
            width:this.props.width,
            height:this.props.height,
            halign : this.props.centered ? "center" : "left",
            fontSize:PcConfig.scenario.fontSizeSmall,
            color: PcConfig.scenario.fontColor,
            showDelay:"0,1s"
          }
        ),
        e('div', {style:{width:"100%", height:100*PcConfig.renderSize}}),
      );

  }

}
