class PcScreenInfos extends PcScreen
{
  constructor(props)
  {
    super(props);

    this.buttons = [];
    this.buttons["previous"] = React.createRef();
    this.buttons["next"] = React.createRef();

    this.form = React.createRef();
    this.alertWindow = React.createRef();
    this.docWindow = React.createRef();

    this.focusTimeout = null;
    this.resetTimeout = null;

    this.registerButton(this.buttons["previous"], PcConfig.screen.infos.previousButton.title);
    this.registerButton(this.buttons["next"], PcConfig.screen.infos.nextButton.title);

    this.state = {alertActive:false, alertWindowTitle:"", docActive:false, docWindowTextFile:""};
  }

  normalizeButtons()
  {
    setTimeout(()=>{this.normalizeButtonsWidth()}, 10);
  }

  componentDidMount()
  {
    if (this.props.active)
    {
      this.normalizeButtons();
      ReactDOM.findDOMNode(this).focus();
    }
  }

  componentDidUpdate()
  {
    if (this.props.active)
    {
      this.normalizeButtons();
      ReactDOM.findDOMNode(this).focus();
    }
  }

  onClickAlertWindow()
  {
    this.setState({alertActive:false});
  }

  onClickDocWindow()
  {
    this.setState({docActive:false});
  }

  clickPrevious()
  {
    if (this.form.current.currentElement==0)
    {
      //PcConfig.sounds.buttonPrevious.play();
      this.props.goPrevScreen();
    }
    else {
      this.form.current.goPrevious();

      let b = this.form.current.getNextButton();
      this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
    }

  }

  clickNext()
  {
    var error = this.form.current.validate();

    if (error!="")
    {
      this.alertWindow.current.updateLabel(error);
      this.setState({alertActive:true});
      this.forceUpdate();
    }
    else if (this.form.current.goNext())
    {
      //PcConfig.sounds.buttonStart.play();
      //this.form.current.reset();
      this.props.goNextScreen();
    }
    else {
      let b = this.form.current.getNextButton();
      this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
    }
  }

  forceNext()
  {
    console.log("FORCE NEXT");
    if (this.form.current.goNext())
    {
      this.props.goNextScreen();
    }
    else
    {
      let b = this.form.current.getNextButton();
      this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
    }
  }

  clickDoc(textFile)
  {
    console.log("clickdoc");
    this.docWindow.current.updateTextFile(textFile);
    this.setState({docActive:true});
    this.forceUpdate();
  }

  reset()
  {
    this.form.current.reset();
  }

  render()
  {
    if (!this.props.active)
    {
      clearTimeout(this.focusTimeout);
      clearTimeout(this.resetTimeout);
      this.state.docActive = false;
      return null;
    }

    if (this.props.active)
    {
      if (PcConfig.getMemory("formShouldReset"))
      {
        PcConfig.setMemory("formShouldReset", false);
        this.resetTimeout = setTimeout(()=>{this.reset();}, 10);
        for (let i=0; i<PcConfig.screen.infos.questions.length; i++)
        {
          PcConfig.setMemory("question"+i, "");
        }
      }
    }

    this.focusTimeout = setTimeout(()=>{ReactDOM.findDOMNode(this).focus(); this.form.current.onKeyPressed("")}, 20);

    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.props.noAnimation?"":this.state.animationStyle
        },
      },
      e(PcLabel,
        {
          label:PcConfig.screen.infos.title,
          title:true,
          width:this.props.width,
          height:this.props.height*(0.1),
          x:0,
          y:this.props.height*0.03,
          fontSize:PcConfig.scenario.fontSizeBig,
          color: PcConfig.scenario.fontColor,
          showDelay:"0,1s"
        }
      ),
      e(PcForm,
        {
          ref:this.form,
          active:this.props.active,
          keepState:this.props.keepState,
          clickPrevious:()=>this.clickPrevious(),
          clickNext:()=>this.clickNext(),
          forceNext:()=>this.forceNext(),
          onClickFormDoc:(textFile)=>this.clickDoc(textFile),
          goNextScreen:this.props.goNextScreen,
          title:PcConfig.screen.infos.title,
          position:"absolute",
          left:this.props.width*(1-PcMode.screen.infos.formW)/2,
          top:this.props.height*0.1,
          width:this.props.width*PcMode.screen.infos.formW,
          height:this.props.height*0.75,
          fontFamily:PcConfig.scenario.fontFamily,
          fontSize:PcConfig.scenario.fontSize,
          color: PcConfig.scenario.fontColor,
          centered : true,
        }
      ),
      e('div',
        {
          style:
            {
              position:"absolute",
              width:"100%",
              top:"85%",
              height:"15%",
              display:"inline-flex",
              justifyContent: "space-evenly",
              alignItems:"center",
              alignContent:"space-around"
            }
        },
        e(PcButton,
          {
            ref:this.buttons["previous"],
            label:PcConfig.screen.infos.previousButton.title,
            align:"right",
            onClick:()=>this.clickPrevious(),
            fontColor:PcConfig.screen.infos.previousButton.fontColor,
            borderColor:PcConfig.screen.infos.previousButton.borderColor,
            backgroundColor:PcConfig.screen.infos.previousButton.backgroundColor,
            prevArrow:true,
            sound : PcConfig.sounds.buttonPrevious,
            showDelay:"0.3s"
          }
        ),
        e(PcButton,
          {
            ref:this.buttons["next"],
            label:PcConfig.screen.infos.questions.length==1 ?  PcConfig.screen.infos.nextButton.titleValidate : PcConfig.screen.infos.nextButton.title,
            fontColor:PcConfig.screen.infos.nextButton.fontColor,
            borderColor:PcConfig.screen.infos.nextButton.borderColor,
            backgroundColor:PcConfig.screen.infos.nextButton.backgroundColor,
            nextArrow:PcConfig.screen.infos.questions.length==1 ? false : true,
            animated:true,
            onClick:()=>this.clickNext(),
            sound : PcConfig.sounds.buttonNext,
            showDelay:"0.4s"
          }
        )
      ),
      e(PcAlert,
        {
          ref:this.alertWindow,
          active:this.state.alertActive,
          onClickButton:()=>this.onClickAlertWindow(),
          label:this.state.alertWindowTitle,
          buttonLabel:PcConfig.screen.infos.popButtonLabel,
        }
      ),
      e(PcDoc,
        {
          ref:this.docWindow,
          active:this.state.docActive,
          onClickButton:()=>this.onClickDocWindow(),
          textFile:this.state.docWindowTextFile,
          buttonLabel:PcConfig.screen.infos.popButtonLabel,
        }
      )
    );
  }
}
