
const electron = require('electron');
const app = electron.app;
const BrowserWindow = electron.BrowserWindow;
const path = require('path');
const { globalShortcut } = require('electron')
const process = require('child_process');

let testMode = false;
global.sharedObj = {testMode: testMode};

let focusOff = false;

let aliveTime = Date.now();
let startTime = Date.now();

let mainWindow, screenWindow;
let focusInterval;

console.clear();

const recorderURL = __dirname + "/recorder/Photocabine.exe";

app.commandLine.appendSwitch('autoplay-policy', 'no-user-gesture-required');

const gotTheLock = app.requestSingleInstanceLock()

if (!gotTheLock) {
  app.quit()
};

function createMainWindow () {

  mainWindow = new BrowserWindow({show:false, frame:testMode, alwaysOnTop:true, backgroundColor: '#000000', width: 1800, height: 1000, webPreferences: {nodeIntegration: true, plugins: true, webSecurity:false}});
  if (!testMode) mainWindow.setKiosk(true);
  if (testMode) mainWindow.webContents.openDevTools();
  let file = path.resolve(app.getAppPath(), 'index.html?lang=0');

  mainWindow.webContents.session.clearCache(()=>console.log("cache cleared"));
  mainWindow.loadURL(file);

  initExternalScreen();
  electron.screen.on('display-added', (e, d) => updateExternalScreen(d, true));
  electron.screen.on('display-removed', (e, d) => updateExternalScreen(d, false));

  globalShortcut.register('Escape', () => {
    if (screenWindow) screenWindow.hide();
    mainWindow.hide();
    closeRecorder();
  })

  globalShortcut.register('CommandOrControl+D', () => {
    mainWindow.webContents.openDevTools();
  })

  mainWindow.on('close', () => {
      if (screenWindow) screenWindow.hide();
  })

  mainWindow.on('closed', () => {
    mainWindow = null;
    screenWindow = null;
  })

  mainWindow.once('ready-to-show', () => {
    mainWindow.show();
  })

  focusInterval = setInterval(()=>{
    if (testMode || focusOff) return;
    mainWindow.setAlwaysOnTop(true);
    mainWindow.setFullScreen(true);
    mainWindow.setKiosk(true);
  }, 5000);
}

function getExternalDisplay()
{
  var displays = electron.screen.getAllDisplays();

  var externalDisplay = null;
  for (var i in displays) {
    if (displays[i].bounds.x != 0 || displays[i].bounds.y != 0) {
      externalDisplay = displays[i];
      break;
    }
  }

  return externalDisplay;
}

function initExternalScreen()
{
  var externalDisplay = getExternalDisplay();

  if (externalDisplay) {
    screenWindow = new BrowserWindow({frame:false, backgroundColor: '#000000',   x: externalDisplay.bounds.x,   y: externalDisplay.bounds.y, width: externalDisplay.size.width, height: externalDisplay.size.height, webPreferences: {nodeIntegration: true, plugins: true, webSecurity:false}});
    screenWindow.loadURL("http://localhost/screen.php");
    screenWindow.setKiosk(true);
  }

}

function updateExternalScreen(d, show)
{
  if (!screenWindow) return;

  if (show)
  {
    var externalDisplay = getExternalDisplay();
    screenWindow.setBounds(externalDisplay.bounds);
    screenWindow.show();
  }
  else {
    screenWindow.hide();
  }
}

function initRecorder()
{
  let cmd =  "\"" + recorderURL + "\" /Init";
  process.exec(cmd);
  createMainWindow();
}

function closeRecorder()
{
  clearTimeout(focusInterval);

  let cmd =  "\"" + recorderURL + "\" /StopRecording";
  process.exec(cmd);

  cmd =  "\"" + recorderURL + "\" /StopLiveView";
  process.exec(cmd);

  cmd =  'Taskkill /IM Photocabine.Process.exe /F /T';
  setTimeout(()=>process.exec(cmd), 1000);
  setTimeout(()=>app.quit(), 2000);
}

app.on('ready', initRecorder);

app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    closeRecorder();
  }
});

app.on('activate', () => {
  if (mainWindow === null) {
    initRecorder();
  }
});


electron.ipcMain.on('alive', (event, arg) => {
	//console.log('alive received at : ' + arg);
  aliveTime = Date.now();
});

electron.ipcMain.on('windowTop', (event, arg) => {
  if (!focusOff) return;

  mainWindow.setAlwaysOnTop(true);
  focusOff = false;
});

electron.ipcMain.on('windowBack', (event, arg) => {
  if (focusOff) return;

  mainWindow.setAlwaysOnTop(false);
  focusOff = true;
});

function checkAliveTime()
{
  if ((Date.now()-aliveTime)>60*1000)
  {
    console.log("ALERT NOT ALIVE : RESTART IN 15 SECONDS");
    clearTimeout(checkAliveTimer);
    setTimeout(()=>{restart()}, 15000);
  }

  if ((Date.now()-startTime)>12*60*60*1000) //running for more than 12 hours
  {
    let d = new Date();
    if (d.getHours()>=5 && d.getHours()<7)
    {
      console.log("ALERT LONG TIME RUNNING : RESTART IN 15 SECONDS");
      clearTimeout(checkAliveTimer);
      setTimeout(()=>{restart()}, 15000);
    }
  }
}

function restart()
{
  clearTimeout(focusInterval);

  let cmd =  "\"" + recorderURL + "\" /StopRecording";
  process.exec(cmd);

  cmd =  "\"" + recorderURL + "\" /StopLiveView";
  process.exec(cmd);

  cmd =  'Taskkill /IM Photocabine.Process.exe /F /T';
  setTimeout(()=>process.exec(cmd), 1000);
  app.relaunch();
  setTimeout(()=>app.quit(), 2000);
}

let checkAliveTimer = setInterval(()=>{checkAliveTime()}, 1000);
